/* (C) Copyright 1998 Florian La Roche
 * for details see file COPYING
 */

/* small sample program to show how to use bootpc.c e.g. within
   a setup program */
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <string.h>
#include "bootpc.h"

/* list of some useful options (see RFC 1533 for more):
	0	padding for alignment
	1	subnet mask
	2	time offset in seconds from UTC
	3	list of routers for this subnet
	4	list of rfc868 time servers
	6	list of DNS name servers
	9	list of rfc1179 printer servers
	12	hostname of client (may be fqdn or leaf)
	15	domain name for use with the DNS
	16	IP address of the swap server
	17	path to mount as root filesystem
	40	NIS domainname
	41	list of NIS servers
	42	list of NTP servers
	48	list of X Window system font servers
	49	list of systems running X Display Manager (xdm)
	255	end cookie
*/

static __u32 netmask, gateway;
static int subnet;

int parse_options (unsigned char *cookie, int cookielength, __u32 ip)
{
	int i = 4, c, end = 0, subnet = 0;
	while (i < cookielength && !end) {
		c = cookie[i];
		if (c == 0)		/* padding */
			++i;
		else if (c == 255)	/* end */
			end = 1;
		else {
			int len = cookie[i + 1] + 2;
			if (i + len > cookielength) return -1;
			if (c == 1 && len == 6) {
				memcpy (&netmask, cookie + i + 2, 4);
				subnet = 1;
			} else if (c == 3 && len >= 6)
	    			memcpy (&gateway, cookie + i + 2, 4);
			i += len;
		}
	}
	return 0;
}

__u32 bootp_server = -1;
int bootp_timeout = 32;

int xdo_bootp(const char *device, __u32 *ip, __u32 *n, __u32 *g)
{
	*ip = netmask = gateway = 0;
	subnet = 0;
	if (do_bootp("eth0", 0, ip) == -1)
		return -1;
	if (!subnet) {
		if ((*ip & 0x80000000) == 0)		/* Class A */
			netmask = htonl (0xFF000000);
		else if ((*ip & 0x40000000) == 0)	/* Class B */
			netmask = htonl (0xFFFF0000);
		else if ((*ip & 0x20000000) == 0)	/* Class C */
			netmask = htonl (0xFFFFFF00);
		else
			netmask = htonl (0xFFFFFF00);
	}
	*n = netmask;
	*g = gateway;
	return 0;
}
