/* (C) Copyright 1998 Florian La Roche
 * for details see file COPYING
 */

#include "dialog.h"

unsigned char dialog_input_result[MAX_LEN + 1];

int dialog_inputbox (const char *title, const char *prompt, int height,
		int width, const char *init)
{
    int i, x, y, box_y, box_x, box_width;
    int input_x = 0, scroll = 0, key = 0, button = -1;
    unsigned char *instr = dialog_input_result;
    WINDOW *dialog;

    /* center dialog box on screen */
    x = (COLS - width) / 2;
    y = (LINES - height) / 2;

    dialog = _dialog_newwin (height, width, y, x, 1);

    wattrset (dialog, border_attr);
    wmove (dialog, height - 3, 0);
    waddch (dialog, ACS_LTEE);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ACS_HLINE);
    wattrset (dialog, dialog_attr);
    waddch (dialog, ACS_RTEE);
    wmove (dialog, height - 2, 1);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ' ');

    _dialog_title (dialog, width, title);
    _dialog_print_autowrap (dialog, prompt, width - 2, 1, 3);

    /* Draw the input field box */
    box_width = width - 6;
    getyx (dialog, y, x);
    box_y = y + 2;
    box_x = (width - box_width) / 2;
    _dialog_draw_box (dialog, y + 1, box_x - 1, 3, box_width + 2,
	      border_attr, dialog_attr);

    x = width / 2 - 11;
    y = height - 2;
    _dialog_print_button (dialog, "Cancel", y, x + 14, FALSE);
    _dialog_print_button (dialog, "  OK  ", y, x, TRUE);

    /* Set up the initial value */
    wmove (dialog, box_y, box_x);
    wattrset (dialog, inputbox_attr);
    if (!init)
	instr[0] = '\0';
    else
	strcpy (instr, init);
    input_x = strlen (instr);
    if (input_x >= box_width) {
	scroll = input_x - box_width + 1;
	input_x = box_width - 1;
	for (i = 0; i < box_width - 1; i++)
	    waddch (dialog, instr[scroll + i]);
    } else
	waddstr (dialog, instr);
    wmove (dialog, box_y, box_x + input_x);

    wrefresh (dialog);
    while (key != ESC) {
	key = wgetch (dialog);

	if (button == -1) {	/* Input box selected */
	    switch (key) {
	    case TAB:
	    case KEY_UP:
	    case KEY_DOWN:
		break;
	    case KEY_LEFT:
		continue;
	    case KEY_RIGHT:
		continue;
	    case KEY_BACKSPACE:
	    case 127:
		if (input_x || scroll) {
		    wattrset (dialog, inputbox_attr);
		    if (!input_x) {
			scroll = scroll < box_width - 1 ?
			    0 : scroll - (box_width - 1);
			wmove (dialog, box_y, box_x);
			for (i = 0; i < box_width; i++)
			    waddch (dialog, instr[scroll + input_x + i] ?
				    instr[scroll + input_x + i] : ' ');
			input_x = strlen (instr) - scroll;
		    } else
			input_x--;
		    instr[scroll + input_x] = '\0';
		    wmove (dialog, box_y, input_x + box_x);
		    waddch (dialog, ' ');
		    wmove (dialog, box_y, input_x + box_x);
		    wrefresh (dialog);
		}
		continue;
	    default:
		if (key < 0x100 && isprint (key)) {
		    if (scroll + input_x < MAX_LEN) {
			wattrset (dialog, inputbox_attr);
			instr[scroll + input_x] = key;
			instr[scroll + input_x + 1] = '\0';
			if (input_x == box_width - 1) {
			    scroll++;
			    wmove (dialog, box_y, box_x);
			    for (i = 0; i < box_width - 1; i++)
				waddch (dialog, instr[scroll + i]);
			} else {
			    wmove (dialog, box_y, input_x++ + box_x);
			    waddch (dialog, key);
			}
			wrefresh (dialog);
		    } else
			flash ();	/* Alarm user about overflow */
		    continue;
		}
	    }
	}
	switch (key) {
	case 'O':
	case 'o':
	    delwin (dialog);
	    return 0;
	case 'C':
	case 'c':
	    delwin (dialog);
	    return 1;
	case KEY_UP:
	case KEY_LEFT:
	    switch (button) {
	    case -1:
		button = 1;	/* Indicates "Cancel" button is selected */
		_dialog_print_button (dialog, "  OK  ", y, x, FALSE);
		_dialog_print_button (dialog, "Cancel", y, x + 14, TRUE);
		wrefresh (dialog);
		break;
	    case 0:
		button = -1;	/* Indicates input box is selected */
		_dialog_print_button (dialog, "Cancel", y, x + 14, FALSE);
		_dialog_print_button (dialog, "  OK  ", y, x, TRUE);
		wmove (dialog, box_y, box_x + input_x);
		wrefresh (dialog);
		break;
	    case 1:
		button = 0;	/* Indicates "OK" button is selected */
		_dialog_print_button (dialog, "Cancel", y, x + 14, FALSE);
		_dialog_print_button (dialog, "  OK  ", y, x, TRUE);
		wrefresh (dialog);
		break;
	    }
	    break;
	case TAB:
	case KEY_DOWN:
	case KEY_RIGHT:
	    switch (button) {
	    case -1:
		button = 0;	/* Indicates "OK" button is selected */
		_dialog_print_button (dialog, "Cancel", y, x + 14, FALSE);
		_dialog_print_button (dialog, "  OK  ", y, x, TRUE);
		wrefresh (dialog);
		break;
	    case 0:
		button = 1;	/* Indicates "Cancel" button is selected */
		_dialog_print_button (dialog, "  OK  ", y, x, FALSE);
		_dialog_print_button (dialog, "Cancel", y, x + 14, TRUE);
		wrefresh (dialog);
		break;
	    case 1:
		button = -1;	/* Indicates input box is selected */
		_dialog_print_button (dialog, "Cancel", y, x + 14, FALSE);
		_dialog_print_button (dialog, "  OK  ", y, x, TRUE);
		wmove (dialog, box_y, box_x + input_x);
		wrefresh (dialog);
		break;
	    }
	    break;
	case ' ':
	case '\n':
	    delwin (dialog);
	    return (button == -1 ? 0 : button);
	case ESC:
	    break;
	}
    }

    delwin (dialog);
    return -1;			/* ESC pressed */
}
