/* (C) Copyright 1998 Florian La Roche
 * for details see file COPYING
 */

#include "dialog.h"
#include <sys/stat.h>
#include <unistd.h>
#include <sys/mman.h>

static void print_page (WINDOW *, int, int);
#if 0
static int get_search_term (WINDOW *, char *, int, int);
#endif
static void print_position (WINDOW *, int, int);

static char *buf;		/* contents of file */
static int filesize;		/* size of displayed file */
static int filepos;		/* current position in file */
static int hscroll;		/* y offset */

static int max_x;		/* max x value in file */

int dialog_textbox (const char *title, const char *file, int height, int width)
{
    int i, x, y, cur_x, cur_y, key = 0, fd;
    struct stat stat;
#if 0
    int fpos, temp, temp1, dir;
    char search_term[MAX_LEN + 1], *tempptr, *found;
#endif
    WINDOW *dialog, *text;

#if 0
    search_term[0] = '\0';
#endif

    if ((fd = open (file, O_RDONLY)) == -1)
	return -1;
    if (fstat (fd, &stat) == -1 || !S_ISREG(stat.st_mode)
		|| ((buf = mmap (NULL, stat.st_size, PROT_READ,
			MAP_SHARED|MAP_DENYWRITE, fd, 0)) == (void*)-1)) {
	close (fd);
	return -1;
    }
    filesize = stat.st_size;
    max_x = cur_x = 0;
    for (i = 0; i < filesize; i++)
	if (buf[i] == '\n')
	    cur_x = 0;
	else {
	    cur_x++;
	    if (max_x < cur_x)
		max_x = cur_x;
	}

    x = (COLS - width) / 2;
    y = (LINES - height) / 2;

    dialog = _dialog_newwin (height, width, y, x, 0);

    text = subwin (dialog, height - 4, width - 2, y + 1, x + 1);
    keypad (text, TRUE);

    _dialog_draw_box (dialog, 0, 0, height, width, dialog_attr, border_attr);

    wattrset (dialog, border_attr);
    wmove (dialog, height - 3, 0);
    waddch (dialog, ACS_LTEE);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ACS_HLINE);
    wattrset (dialog, dialog_attr);
    waddch (dialog, ACS_RTEE);
    wmove (dialog, height - 2, 1);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ' ');

    _dialog_title (dialog, width, title);
    _dialog_print_button (dialog, " EXIT ", height - 2, width / 2 - 4, TRUE);
    wnoutrefresh (dialog);

    getyx (dialog, cur_y, cur_x);
    _dialog_attr_clear (text, height - 4, width - 2, dialog_attr);
    filepos = hscroll = 0;
    print_page (text, height - 4, width - 2);
    print_position (dialog, height, width);
    wmove (dialog, cur_y, cur_x);
    wrefresh (dialog);

    while ((key != ESC) && (key != '\n')) {
	key = wgetch (dialog);
	switch (key) {
	case 'E': case 'e':
	    delwin (dialog);
	    munmap (buf, stat.st_size);
	    close (fd);
	    return 0;
	case 'g': case KEY_HOME:
	    if (! filepos)
		break;
	    filepos = hscroll = 0;
	    print_page (text, height - 4, width - 2);
	    print_position (dialog, height, width);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
	case 'G': case KEY_END:
	    filepos = filesize? filesize - 1 : 0;
	    hscroll = 0;
	    for (i = height - 4; filepos && i; ) {
		if (buf[filepos-1] == '\n' && !--i)
		    break;
		filepos--;
	    }
	    print_page (text, height - 4, width - 2);
	    print_position (dialog, height, width);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
	case 'K': case 'k': case KEY_UP:
	    if (! filepos)
		break;
	    while (filepos && buf[--filepos-1] != '\n')
		/* nothing */ ;
	    print_page (text, height - 4, width - 2);
	    print_position (dialog, height, width);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
	case 'B': case 'b': case KEY_PPAGE:
	    if (! filepos)
		break;
	    for (i = height - 4; filepos && i; ) {
		if (buf[filepos-1] == '\n' && !--i)
		    break;
		filepos--;
	    }
	    print_page (text, height - 4, width - 2);
	    print_position (dialog, height, width);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
	case 'J': case 'j': case KEY_DOWN:
	    while (filepos < filesize && buf[filepos++] != '\n')
		;
	    print_page (text, height - 4, width - 2);
	    print_position (dialog, height, width);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
	case ' ': case KEY_NPAGE:
	    for (i = height - 4; i--; )
		while (filepos < filesize && buf[filepos++] != '\n')
		    ;
	    print_page (text, height - 4, width - 2);
	    print_position (dialog, height, width);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
	case '0': case 'H': case 'h': case KEY_LEFT:
	    if (hscroll <= 0)
		break;
	    if (key == '0')
		hscroll = 0;
	    else
		hscroll--;
	    print_page (text, height - 4, width - 2);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
	case 'L': case 'l': case KEY_RIGHT:
	    if (hscroll >= MAX_LEN)
		break;
	    hscroll++;
	    print_page (text, height - 4, width - 2);
	    wmove (dialog, cur_y, cur_x);
	    wrefresh (dialog);
	    break;
#if 0
	case '/': case 'n': case '?': case 'N':
	    dir = (key == '/' || key == 'n') ? 1 : 0;
	    if (dir ? !end_reached : !begin_reached) {
		if (key == 'n' || key == 'N') {
		    if (search_term[0] == '\0') {
			fprintf (stderr, "\a");
			break;
		    }
		} else if (get_search_term (text, search_term, height - 4,
					 width - 2) == -1) {
		    /* ESC pressed in get_search_term().
		       Reprint page to clear box */
		    wattrset (text, dialog_attr);
		    back_lines (page_length);
		    print_page (text, height - 4, width - 2);
		    wmove (dialog, cur_y, cur_x);
		    wrefresh (dialog);
		    break;
		}
		/* Save variables for restoring in case search term
		   can't be found */
		tempptr = page;
		temp = begin_reached;
		temp1 = end_reached;
		if ((fpos = lseek (fd, 0, SEEK_CUR)) == -1) {
		    endwin ();
		    fprintf (stderr, "\nError moving file pointer in "
			     "dialog_textbox().\n");
		    exit (-1);
		}
		fpos -= bytes_read;
		/* update 'page' to point to next (previous) line before
		   forward (backward) searching */
		back_lines (dir ? page_length - 1 : page_length + 1);
		found = NULL;
		if (dir)	/* Forward search */
		    while ((found = strstr (get_line (), search_term))
			   == NULL) {
			if (end_reached)
			    break;
		} else		/* Backward search */
		    while ((found = strstr (get_line (), search_term))
			   == NULL) {
			if (begin_reached)
			    break;
			back_lines (2);
		    }
		if (found == NULL) {	/* not found */
		    fprintf (stderr, "\a");	/* beep */
		    /* Restore program state to that before searching */
		    if (lseek (fd, fpos, SEEK_SET) == -1) {
			endwin ();
			fprintf (stderr, "\nError moving file pointer in "
				 "dialog_textbox().\n");
			exit (-1);
		    }
		    if ((bytes_read = read (fd, buf, BUF_SIZE)) == -1) {
			endwin ();
			fprintf (stderr, "\nError reading file in "
				 "dialog_textbox().\n");
			exit (-1);
		    }
		    buf[bytes_read] = '\0';
		    page = tempptr;
		    begin_reached = temp;
		    end_reached = temp1;
		    /* move 'page' to point to start of current page in order to
		       re-print current page. Note that 'page' always points to
		       start of next page, so this is necessary */
		    back_lines (page_length);
		} else		/* Search term found */
		    back_lines (1);
		/* Reprint page */
		wattrset (text, dialog_attr);
		print_page (text, height - 4, width - 2);
		if (found != NULL)
		    print_position (dialog, height, width);
		wmove (dialog, cur_y, cur_x);	/* Restore cursor position */
		wrefresh (dialog);
	    } else		/* no need to find */
		fprintf (stderr, "\a");		/* beep */
	    break;
#endif
	case ESC:
	    break;
	}
    }
    delwin (dialog);
    munmap (buf, stat.st_size);
    close (fd);
    return -1;
}

static void print_page (WINDOW * win, int height, int width)
{
    int n, x, y, i = filepos;

    for (y = 0; y < height; y++) {
	for (n = hscroll; n--;)
		if (i < filesize && buf[i] != '\n')
			i++;
	wmove (win, y, 0);
	waddch (win, ' ');
	for (x = width - 2; x;) {
		if (i >= filesize || buf[i] == '\n')
			break;
		if (1 || isprint((unsigned char)buf[i])) {
			waddch (win, buf[i]);
			x--;
		}
		i++;
	}
	for (++x; x--;)
		waddch (win, ' ');
	while (i < filesize && buf[i] != '\n')
		i++;
	if (i < filesize)
		i++;
    }
    wnoutrefresh (win);
}

#if 0
static int get_search_term (WINDOW * win, char *search_term, int height,
		int width)
{
    int i, x, y, input_x = 0, scroll = 0, key = 0;
    int box_height = 3, box_width = 30;

    x = (width - box_width) / 2;
    y = (height - box_height) / 2;
#ifdef HAVE_SHADOW
    _dialog_draw_shadow (win, y, x, box_height, box_width);
#endif
    _dialog_draw_box (win, y, x, box_height, box_width, dialog_attr,
	      searchbox_border_attr);
    wattrset (win, searchbox_title_attr);
    wmove (win, y, x + box_width / 2 - 4);
    waddstr (win, " Search ");

    box_width -= 2;
    wmove (win, y + 1, x + 1);
    wrefresh (win);
    search_term[0] = '\0';
    wattrset (win, searchbox_attr);
    while (key != ESC) {
	key = wgetch (win);
	switch (key) {
	case '\n':
	    if (search_term[0] != '\0')
		return 0;
	    break;
	case KEY_BACKSPACE:
	case 127:
	    if (input_x || scroll) {
		if (!input_x) {
		    scroll = scroll < box_width - 1 ?
			0 : scroll - (box_width - 1);
		    wmove (win, y + 1, x + 1);
		    for (i = 0; i < box_width; i++)
			waddch (win, search_term[scroll + input_x + i] ?
				search_term[scroll + input_x + i] : ' ');
		    input_x = strlen (search_term) - scroll;
		} else
		    input_x--;
		search_term[scroll + input_x] = '\0';
		wmove (win, y + 1, input_x + x + 1);
		waddch (win, ' ');
		wmove (win, y + 1, input_x + x + 1);
		wrefresh (win);
	    }
	    break;
	case ESC:
	    break;
	default:
	    if (isprint (key))
		if (scroll + input_x < MAX_LEN) {
		    search_term[scroll + input_x] = key;
		    search_term[scroll + input_x + 1] = '\0';
		    if (input_x == box_width - 1) {
			scroll++;
			wmove (win, y + 1, x + 1);
			for (i = 0; i < box_width - 1; i++)
			    waddch (win, search_term[scroll + i]);
		    } else {
			wmove (win, y + 1, input_x++ + x + 1);
			waddch (win, key);
		    }
		    wrefresh (win);
		}
	}
    }

    return -1;
}
#endif

static void print_position (WINDOW * win, int height, int width)
{
    int percent;

    wattrset (win, position_indicator_attr);
    percent = !filesize ?  100 : (filepos * 100) / filesize;
    wmove (win, height - 3, width - 9);
    wprintw (win, "(%3d%%)", percent);
}
