/* (C) Copyright 1998 Florian La Roche
 * for details see file COPYING
 */

#include "dialog.h"

int dialog_yesno (const char *title, const char *prompt, int height, int width)
{
    int i, x, y, key = 0, button = 0;
    WINDOW *dialog;

    x = (COLS - width) / 2;
    y = (LINES - height) / 2;

    dialog = _dialog_newwin (height, width, y, x, 1);

    wattrset (dialog, border_attr);
    wmove (dialog, height - 3, 0);
    waddch (dialog, ACS_LTEE);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ACS_HLINE);
    wattrset (dialog, dialog_attr);
    waddch (dialog, ACS_RTEE);
    wmove (dialog, height - 2, 1);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ' ');

    if (title != NULL) {
	wattrset (dialog, title_attr);
	wmove (dialog, 0, (width - strlen (title)) / 2 - 1);
	waddch (dialog, ' ');
	waddstr (dialog, title);
	waddch (dialog, ' ');
    }
    _dialog_print_autowrap (dialog, prompt, width - 2, 1, 3);

    x = width / 2 - 10;
    y = height - 2;
    _dialog_print_button (dialog, "  No  ", y, x + 13, FALSE);
    _dialog_print_button (dialog, " Yes ", y, x, TRUE);
    wrefresh (dialog);

    while (key != ESC) {
	key = wgetch (dialog);
	switch (key) {
	case 'Y':
	case 'y':
	    delwin (dialog);
	    return 0;
	case 'N':
	case 'n':
	    delwin (dialog);
	    return 1;

	case TAB:
	case KEY_UP:
	case KEY_DOWN:
	case KEY_LEFT:
	case KEY_RIGHT:
	    if (!button) {
		button = 1;	/* "No" button selected */
		_dialog_print_button (dialog, " Yes ", y, x, FALSE);
		_dialog_print_button (dialog, "  No  ", y, x + 13, TRUE);
	    } else {
		button = 0;	/* "Yes" button selected */
		_dialog_print_button (dialog, "  No  ", y, x + 13, FALSE);
		_dialog_print_button (dialog, " Yes ", y, x, TRUE);
	    }
	    wrefresh (dialog);
	    break;
	case ' ':
	case '\n':
	    delwin (dialog);
	    return button;
	case ESC:
	    break;
	}
    }

    delwin (dialog);
    return -1;
}
